#include <map>
#include <limits>
#include <sstream>
#include <stdlib.h> // random(), srandom()
#include <sys/time.h> // time(), to see srandom()
#include <iostream>

#include "fudgeish.hpp"

#ifndef CERR
#define CERR std::cerr << __FILE__ << ":" << std::dec << __LINE__ << " : "
#endif

#ifndef COUT
#define COUT std::cout << __FILE__ << ":" << std::dec << __LINE__ << " : "
#endif





namespace fudge
{
        using namespace std;

	short rollDF()
	{
		static bool seeded = false;
		if( ! seeded )
		{
			seeded = true;
			::srandom( ::time(NULL) );
		}
		return (short) ((::random() % 3) - 1);
	}


        int rollContest( int difficultyRating, int skillRating )
        {

                static DiceBag dice( 4 );
                int roll = dice.roll();
                COUT << "rollContest( " << difficultyRating << ", " << skillRating << " ): " << dice << std::endl;
                return roll + (skillRating - difficultyRating);
        }

        std::string ratingName( int rating )
        {
                static std::map<int,std::string> namesmap;
                static bool inited = false;
                if( ! inited && (inited=true) )
                {
                        namesmap[RatingTerrible] = "Terrible";
                        namesmap[RatingPoor] = "Poor";
                        namesmap[RatingMediocre] = "Mediocre";
                        namesmap[RatingFair] = "Fair";
                        namesmap[RatingGood] = "Good";
                        namesmap[RatingGreat] = "Great";
                        namesmap[RatingSuperb] = "Superb";
                        //namesmap[RatingSuperb+1] = "Legendary";
                }
                std::string name;
                if( rating < RatingTerrible )
                {
			std::ostringstream os;
			os << namesmap[RatingTerrible] << "-" << ( -1 * (rating  - RatingTerrible) );
                        name = os.str();
                }
                else if ( rating > RatingSuperb )
                {
			std::ostringstream os;
			os <<  namesmap[RatingSuperb] << "+" << ( rating - RatingSuperb );
                        name = os.str();
                }
                else name = namesmap[rating];

                return name;
        }

        std::ostream & operator<<( std::ostream & os, const fudge::Buff & ca )
        {
                os << "[" << ca.name();
                os << " x"<< ca.count();
                os << "]";
                os << "] Scale="<<ca.scale();
//                         os << " x"<< ca->count();
                if( ca.offensive() ) os <<" ODF="<<fudge::ratingName(ca.odf());
                if( ca.defensive() ) os <<" DDF="<<fudge::ratingName(ca.ddf());
                // << " Range(?)="<< fudge::ratingName(ca.range());
                return os;
        };


        dF::dF( short whichside )
        {
                this->side( whichside );
        };

        short
        dF::roll()
        {
                return this->m_side = rollDF();
        }
        std::ostream &
        operator<<( std::ostream & os, const dF & hp )
        {
                return os << (hp.side() == 0 ? "0" : (hp.side()<0 ? "-" : "+") );
        };

        std::istream &
        operator>>( std::istream & is, dF & hp )
        {
                char c;
                is >> c;
                switch( c )
                {
                        case '+': hp.side(1); break;
                        case '-': hp.side(-1); break;
                        default: hp.side(0); break;
                }
                return is;
        };


        char dF::symbol() const
        {
                switch( this->m_side )
                {
                  case -1: return '-';
                  case 1: return '+';
                  default: return '0';
                }
        }



        DiceBag::DiceBag( int dicecount ) : m_lastroll(0)
        {
                this->diceCount( dicecount );
                //this->m_dice.autoDelete( true );
        }

        DiceBag::list_type &
        DiceBag::dice()
        {
                return this->m_dice;
        }

        const DiceBag::list_type &
        DiceBag::dice() const
        {
                return this->m_dice;
        }


//         bool
//         DiceBag::operator==( const DiceBag &rhs )
//         {
//                 DiceBag::const_iterator tit = this->dice().begin();
//                 DiceBag::const_iterator it;
//                 for( it = rhs.dice().begin(); it != rhs.dice.end() && tit != this->dice().end(); it++, tit++ )
//                 {
//                         if ! ((*it) == (*tit)) return false;
//                 }
//                 return true;
//         }

//         bool
//         DiceBag::operator>( const DiceBag & rhs )
//         {
//                 DiceBag::const_iterator tit = this->dice().begin();
//                 DiceBag::const_iterator it;
//                 for( it = rhs.dice().begin(); it != rhs.dice.end() && tit != this->dice().end(); it++, tit++ )
//                 {
//                         if( (*it) > (*tit) ) return false;
//                 }
//                 return true;
//         }
//         bool
//         DiceBag::operator<( const DiceBag & rhs )
//         {
//                 bool ret = true;
//                 DiceBag::const_iterator tit = this->dice().begin();
//                 DiceBag::const_iterator it;
//                 for( it = rhs.dice().begin(); it != rhs.dice.end() && tit != this->dice().end(); it++, tit++ )
//                 {
//                         if( (*it) > (*tit) ) return false;
//                         os << (*it);
//                 }
//                 return ret;
//         }


        std::ostream & operator<<( std::ostream & os, const DiceBag::list_type & dice )
        {
                DiceBag::const_iterator it;
                for( it = dice.begin(); it != dice.end(); it++ )
                {
                        os << (*it);
                }
                return os;
        }
        std::ostream &
        operator<<( std::ostream & os, const DiceBag & db )
        {
                int dcount = db.m_dice.size();
                os << dcount<<"dF [";
                os << db.m_dice;
                os << "] = " <<db.m_lastroll;
                return os;
        };



        void
        DiceBag::diceCount( int c )
        {
                this->m_dice.clear();
		this->m_dice.reserve(c);
                for( int i = 0; i < c; i++ )
                {
                        this->m_dice.push_back( dF() ); // new dF() );
                }
        }

        int
        DiceBag::diceCount() const
        {
                return this->m_dice.size();
        }

        int // static
        DiceBag::roll( int howmany, list_type & thedice )
        {
                int total = 0;
                int roll = 0;
                for( int i = 0; i < howmany; i++ )
                {
                        roll = rollDF();
                        // COUT << "roll="<<roll<<endl;
                        thedice.push_back( dF( roll ) ); // new dF(roll) );
                        total += roll;
                }
                return total;
        };

        int 
        DiceBag::roll()
        {
                int s = this->diceCount();
                this->m_dice.clear();
                return this->m_lastroll = roll( s, this->m_dice );
        }


        WoundChart::WoundChart( int MaxScratch,
                                int MaxHurt,
                                int MaxVeryHurt,
                                int MaxIncap,
                                int MaxNearDeath )
        {
                this->init();
                m_boxes[NoEffect] = 0;
                m_boxes[Scratch] = MaxScratch;
                m_boxes[Hurt] = MaxHurt;
                m_boxes[VeryHurt] = MaxVeryHurt;
                m_boxes[Incap] = MaxIncap;
                m_boxes[NearDeath] = MaxNearDeath;
        }

// #if FUDGE_USE_S11N
//         bool
//         WoundChart::s7e( s11n::s11n_node & node ) const
//         {
//                 if( ! this->serializable::s7e( node ) ) return false;
//                 return true;
//         }

//         bool
//         WoundChart::d9e( const s11n::s11n_node &node )
//         {
//                 if( ! this->serializable::d9e( node ) ) return false;
//                 return true;
//         }
// #endif // FUDGE_USE_S11N



        void
        WoundChart::damageConversion( int MinScratch, int MinHurt, int MinVH, int MinIncap, int MinND, int MinDead )
        {
                m_dmgchart[Scratch] = MinScratch;
                m_dmgchart[Hurt] = MinHurt;
                m_dmgchart[VeryHurt] = MinVH;
                m_dmgchart[Incap] = MinIncap;
                m_dmgchart[NearDeath] = MinND;
                m_dmgchart[Dead] = (MinDead == -1 ? (MinND+MinND-MinIncap) : MinDead);
        }

        int
        WoundChart::wound( int severity )
        {
                if( severity == Dead )
                {
                        ++this->m_hits[Dead];
                        return Dead;
                }
                if( (NearDeath < severity) || (NoEffect >= severity) ) return -1; // bounds check for m_{hits,boxes}
                int hits = m_hits[severity];
                if( hits >= this->m_boxes[severity] )
                { // Your slots are full. Rolling over to the next higher wound category :(
                        if( this->m_migratewounds )
                        {
                                if( severity == NearDeath )
                                {
                                        // You're dead, already!
                                        ++this->m_hits[Dead];
                                        return Dead;
                                }
                                //if( this->m_boxes[severity] == 0 )
                                return this->wound( severity + 1 );
                        }
                }
                else
                { // chalk up a bruise...
                        ++m_hits[severity];
                }
                return severity;
        }

        int
        WoundChart::severity( int dmg )
        {
                for( int i = WoundChart::Scratch; i <= WoundChart::Dead; i++ )
                {
                        if( dmg < m_dmgchart[i] ) return i - 1;
                }
//                 if( dmg < m_dmgchart[Scratch] ) return -1;
//                 if( dmg < m_dmgchart[Hurt] ) return Scratch;
//                 if( dmg < m_dmgchart[VeryHurt] ) return Hurt;
//                 if( dmg < m_dmgchart[Incap] ) return VeryHurt;
//                 if( dmg < m_dmgchart[NearDeath] ) return Incap;
//                 if( dmg < m_dmgchart[Dead] ) return NearDeath;
                return Dead; // reasonable default, eh?
        }

        int
        WoundChart::applyDamage( int dmg )
        {
                return this->wound( this->severity( dmg ) );
        }


        bool
        WoundChart::isFull()
        {
                for( int i = Scratch; i <= NearDeath; i++ ) // intentionally does not count Dead hits.
                {
                        if( this->m_hits[i] < this->m_boxes[i] ) return false;
                }
                return true;
        }

        bool
        WoundChart::isDead()
        {
                return this->m_hits[WoundChart::Dead] > 0;
        }

        std::string
        WoundChart::woundName( int severity )
        {
                static std::map<int,std::string> namesmap;
                static bool inited = false;
                if( severity < WoundChart::NoEffect ) severity = WoundChart::NoEffect;
                else if( severity > WoundChart::Dead ) severity = WoundChart::Dead;
                if( ! inited && (inited=true) )
                {
                        // note: the removal of spaces here is to
                        // accomodate using these as key names in
                        // s11n'd data :/
                        namesmap[WoundChart::NoEffect] = "NoEffect";
                        namesmap[WoundChart::Scratch] = "Scratch";
                        namesmap[WoundChart::Hurt] = "Hurt";
                        namesmap[WoundChart::VeryHurt] = "VeryHurt";
                        namesmap[WoundChart::Incap] = "Incapacitated";
                        namesmap[WoundChart::NearDeath] = "NearDeath";
                        namesmap[WoundChart::Dead] = "Dead";
                }
                return namesmap[severity];
        }
                
        std::ostream & operator<<( std::ostream & os, const WoundChart &rhs )
        {
                //os << "Wounds chart:" << std::endl;
                for( int i = WoundChart::Scratch; i <= WoundChart::Dead; i++ )
                {
                        os << "["<<rhs.m_boxes[i]<<"] ";
                        os <<WoundChart::woundName(i);
                        os << ": ";
                        for( int h = 0; h < rhs.m_hits[i]; h++ )
                        {
                                os << "X";
                        }
                        os << std::endl;
                }
                //                os << "Deaths: " << rhs.m_hits[WoundChart::Dead] << std::endl;
                return os;
        };
                

        void
        WoundChart::clear()
        {
                for( int i = Scratch; i <= Dead; i++ )
                {
                        m_hits[i] = 0;
                }
        }

        void
        WoundChart::init()
        {
                this->m_migratewounds = true;
                m_hits = vector_type(Dead+1);
                m_boxes = vector_type(Dead+1);
                m_dmgchart = vector_type(Dead+1);
                for( int i = Scratch; i <= NearDeath; i++ )
                {
                        m_hits[i] = 0;
                        m_boxes[i] = 0;
                }
                m_boxes[Dead] = 1; // std::numeric_limits<int>::max();
                m_hits[Dead] = 0;
                this->damageConversion( 1,3,5,7,9 );
                // note that the boxes for Dead is not honored, but
                // this is intentional, as Dead is very vague in
                // Fudge.
        };


}; // namespace fudge

