#ifndef fudge_FUDGE_S11N_HPP_INCLUDED
#define fudge_FUDGE_S11N_HPP_INCLUDED 1

#include "fudgeish.hpp"
#include <s11n.net/s11n/s11nlite.hpp>
#include <s11n.net/s11n/list.hpp>

namespace fudge {


        struct dF_s11n
        {
                template <typename NodeT>
                bool operator()( NodeT & node, const dF & src ) const
                {
                        typedef s11n::node_traits<NodeT> NTR;
                        NTR::class_name( node, "fudge::dF" );
                        NTR::set( node, "side", (src.side() == 0 ? "0" : (src.side()<0 ? "-" : "+") ) );
                        return true;
                }

                template <typename NodeT>
                bool operator()( const NodeT & node, dF & tgt ) const
                {
                        typedef s11n::node_traits<NodeT> NTR;
                        std::string s = NTR::get( node, "side", std::string("") );
                        if ( "+" == s ) tgt.side( 1 );
                        else if ( "-" == s ) tgt.side( -1 );
                        else if ( "0" == s ) tgt.side( 0 );
                        // else don't touch it.
                        return true;
                }
        };

        struct DiceBag_s11n
        {
                template <typename NodeT>
                bool operator()( NodeT & node, const DiceBag & src ) const
                {
                        typedef s11n::node_traits<NodeT> NTR;
                        NTR::class_name( node, "fudge::DiceBag" );
                        DiceBag::list_type::const_iterator dit = src.dice().begin();
                        DiceBag::list_type::const_iterator det = src.dice().end();
                        std::string dice;
                        for( ; dit != det; ++dit )
                        {
                                dice += (*dit).symbol();
                        }
                        NTR::set( node, "dice", dice );
                        return true;
                }

                template <typename NodeT>
                bool operator()( const NodeT & node, DiceBag & tgt ) const
                {
                        typedef s11n::node_traits<NodeT> NTR;
                        dF die;
                        char c;
                        std::string dice = NTR::get( node, "dice", "" );
                        for( std::string::size_type i = 0; i < dice.size(); i++ )
                        {
                                c = dice[i];
                                die.side( ('+' == c) ? 1 : (('-' == c) ? -1 : 0) );
                                tgt.dice().push_back( die );
                        }
                        return true;
                }
        };


        struct WoundTrack_s11n
        {
                template <typename NodeT>
                bool operator()( NodeT & node, const WoundTrack & src ) const
                {
                        typedef s11n::node_traits<NodeT> NTR;
                        NTR::class_name( node, "fudge::WoundTrack" );
                        s11n::list::serialize_list( "hits", src.hits() );
                        s11n::list::serialize_list( "boxes", src.boxes() );
                        s11n::list::serialize_list( "damage", src.damageChart() );
                        NTR::set( node, "migrate_wounds", this->migrateWounds() );
                        return true;
                }

                template <typename NodeT>
                bool operator()( const NodeT & node, WoundTrack & tgt ) const
                {
                        typedef s11n::node_traits<NodeT> NTR;
                        s11n::list::deserialize_list( "hits", tgt.hits() );
                        s11n::list::deserialize_list( "boxes", tgt.boxes() );
                        s11n::list::deserialize_list( "damage", tgt.damageChart() );
                        tgt.migrateWounds( NTR::get( node, "migrate_wounds", tgt.migrateWounds() ) );
                        return true;
                }
        };



} // namespace fudge

#define S11N_TYPE fudge::dF
#define S11N_NAME "fudge::dF"
#define S11N_SERIALIZE_FUNCTOR fudge::dF_s11n
#include <s11n.net/s11n/reg_proxy.hpp>

#define S11N_TYPE fudge::DiceBag
#define S11N_NAME "fudge::DiceBag"
#define S11N_SERIALIZE_FUNCTOR fudge::DiceBag_s11n
#include <s11n.net/s11n/reg_proxy.hpp>

#define S11N_TYPE fudge::WoundTrack
#define S11N_NAME "fudge::WoundTrack"
#define S11N_SERIALIZE_FUNCTOR fudge::WoundTrack_s11n
#include <s11n.net/s11n/reg_proxy.hpp>


#endif // fudge_FUDGE_S11N_HPP_INCLUDED
