#ifndef fudge_FUDGE_S11N_1_1_HPP_INCLUDED
#define fudge_FUDGE_S11N_1_1_HPP_INCLUDED 1

#include "fudgeish.hpp"
#include <s11n.net/s11n/s11n.hpp>
#include <s11n.net/s11n/proxy/std_vector.hpp>
#include <s11n.net/s11n/proxy/pod_int.hpp>



namespace fudge {

        struct dF_s11n
        {
                template <typename NodeT>
                bool operator()( NodeT & node, const dF & src ) const
                {
                        typedef s11n::node_traits<NodeT> NTR;
                        NTR::class_name( node, "fudge::dF" );
                        NTR::set( node, "side", (src.side() == 0 ? "0" : (src.side()<0 ? "-" : "+") ) );
                        return true;
                }

                template <typename NodeT>
                bool operator()( const NodeT & node, dF & tgt ) const
                {
                        typedef s11n::node_traits<NodeT> NTR;
                        std::string s = NTR::get( node, "side", std::string("") );
                        if ( "+" == s ) tgt.side( 1 );
                        else if ( "-" == s ) tgt.side( -1 );
                        else if ( "0" == s ) tgt.side( 0 );
                        return true;
                }
        };

        struct DiceBag_s11n
        {
                template <typename NodeT>
                bool operator()( NodeT & node, const DiceBag & src ) const
                {
                        typedef s11n::node_traits<NodeT> NTR;
                        NTR::class_name( node, "fudge::DiceBag" );
                        DiceBag::list_type::const_iterator dit = src.dice().begin();
                        DiceBag::list_type::const_iterator det = src.dice().end();
                        std::string dice;
                        for( ; dit != det; ++dit )
                        {
                                dice += (*dit).symbol();
                        }
                        NTR::set( node, "dice", dice );
                        return true;
                }

                template <typename NodeT>
                bool operator()( const NodeT & node, DiceBag & tgt ) const
                {
                        typedef s11n::node_traits<NodeT> NTR;
                        dF die;
                        char c;
                        std::string dice = NTR::get( node, "dice", "" );
                        for( std::string::size_type i = 0; i < dice.size(); i++ )
                        {
                                c = dice[i];
                                die.side( ('+' == c) ? 1 : (('-' == c) ? -1 : 0) );
                                tgt.dice().push_back( die );
                        }
                        return true;
                }
        };


        struct WoundChart_s11n
        {
                template <typename NodeT>
                bool operator()( NodeT & node, const WoundChart & src ) const
                {
                        typedef s11n::node_traits<NodeT> NTR;
                        NTR::class_name( node, "fudge::WoundChart" );
			typedef WoundChart::vector_type VT;
                        s11n::list::serialize_streamable_list<NodeT,VT>( node, "hits", src.hits() );
                        s11n::list::serialize_streamable_list<NodeT,VT>( node, "boxes", src.boxes() );
                        s11n::list::serialize_streamable_list<NodeT,VT>( node, "damage", src.damageChart() );
                        NTR::set( node, "migrate_wounds", src.migrateWounds() );
                        return true;
                }

                template <typename NodeT>
                bool operator()( const NodeT & node, WoundChart & tgt ) const
                {
                        typedef s11n::node_traits<NodeT> NTR;
			typedef WoundChart::vector_type VT;
                        s11n::list::deserialize_streamable_list<NodeT,VT>( node, "hits", tgt.hits() );
                        s11n::list::deserialize_streamable_list<NodeT,VT>( node, "boxes", tgt.boxes() );
                        s11n::list::deserialize_streamable_list<NodeT,VT>( node, "damage", tgt.damageChart() );
                        tgt.migrateWounds( NTR::get( node, "migrate_wounds", tgt.migrateWounds() ) );
                        return true;
                }
        };


} // namespace fudge

#define S11N_TYPE fudge::dF
#define S11N_TYPE_NAME "fudge::dF"
#define S11N_SERIALIZE_FUNCTOR fudge::dF_s11n
#include <s11n.net/s11n/reg_s11n_traits.hpp>

#define S11N_TYPE fudge::DiceBag
#define S11N_TYPE_NAME "fudge::DiceBag"
#define S11N_SERIALIZE_FUNCTOR fudge::DiceBag_s11n
#include <s11n.net/s11n/reg_s11n_traits.hpp>

#define S11N_TYPE fudge::WoundChart
#define S11N_TYPE_NAME "fudge::WoundChart"
#define S11N_SERIALIZE_FUNCTOR fudge::WoundChart_s11n
#include <s11n.net/s11n/reg_s11n_traits.hpp>

#endif // fudge_FUDGE_S11N_1_1_HPP_INCLUDED
